const Pedidos = require("../infra/repository/pedidos.repository");
const { trocarAno } = require("../util/dateFormat");
const pedidos = new Pedidos();

class pedidosController {
 

    /**
     * Registra um usuário
     * @returns mensagem referente ao resultado obtido
     */
    register() {
        return function (req, res) {
            const  data  = req.body;
            
            pedidos.register(data).then((response) => {
                if (response.error) return res.status(response.code).send({ error: response.error });
                else return res.status(response.code).json(response);
            }).catch((error) => {
                return res.status(400).send({ error: `Não foi possível realizar o cadastro!` });
            });
        };
    }

   
    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
     list() {
        return function (req, res) {
            const  id_user  = req.query.id_user;
            const  data  = req.query.data;
            const  hora  = req.query.hora;

            pedidos.list(id_user, data, hora).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    listFormSocket(dataInicio, dataFim) {
        return new Promise(async (resolve, reject) => {
    
          if (!dataInicio || !dataFim) return resolve({ error: `Data não atribuida!`, code: 203 })
    
        //   dataInicio = trocarAno(dataInicio)
        //   dataFim = trocarAno(dataFim)
    
          db.query(
            `WITH recursive AllDates AS (
            SELECT DATE(?) AS data -- Início do intervalo
            UNION ALL
            SELECT data + INTERVAL 1 DAY
            FROM AllDates
            WHERE data + INTERVAL 1 DAY <= DATE(?)
            )
            SELECT
            DATE_FORMAT(AllDates.data, "%d/%m") AS data,
            COALESCE(SUM(CASE WHEN vendedores.contrato = 'Representante' THEN pedidos.quantidade ELSE 0 END), 0) AS total_representante,
            COALESCE(SUM(pedidos.quantidade), 0) AS total_sem_vinculo,
            SUM(COALESCE(SUM(pedidos.quantidade), 0)) OVER (ORDER BY AllDates.data) AS total_vendas_acumulado
            FROM 
            AllDates
            LEFT JOIN 
            pedidos ON DATE(pedidos.data) = AllDates.data
            LEFT JOIN 
            user ON pedidos.id_user = user.id
            LEFT JOIN 
            vendedores ON user.id_vendedor = vendedores.id
            WHERE 
            pedidos.concluido = 1 OR pedidos.concluido IS NULL
            GROUP BY 
            AllDates.data
            ORDER BY 
            AllDates.data
            LIMIT 366`,
            [dataInicio, dataFim],
            async (error, user) => {

              if (error) return reject(new Error(error));
              return resolve(user);
            }
          );
        });
      }
       
    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listForm() {
        return function (req, res) {
            const dataInicio = req.query.dataInicio;
            const dataFim = req.query.dataFim;

            pedidos.listForm(dataInicio, dataFim).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

        /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listFormByPeriodo() {
        return function (req, res) {
            const periodo = req.query.periodo
            const id_user = req.query.id_user;

            pedidos.listFormByPeriodo(periodo, id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

        /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
        listPedidosConcluidos() {
            return function (req, res) {
                const { id } = req.params;
                const id_user = req.query.id_user;
    
                pedidos.listPedidosConcluidos(id, id_user).then((response) => {
                    if (response.error) res.status(response.code).send({ error: response.error });
                    else return res.status(200).send(response);
                }).catch((error) => {
                    throw new Error(error);
                });
            }
        }

        
    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
        listPedidosPendentes() {
            return function (req, res) {
                const { id } = req.params;
                const id_user = req.query.id_user;
    
                pedidos.listPedidosPendentes(id, id_user).then((response) => {
                    if (response.error) res.status(response.code).send({ error: response.error });
                    else return res.status(200).send(response);
                }).catch((error) => {
                    throw new Error(error);
                });
            }
        }

    /**
     * Lista todos os usuarios
     * @returns mensagem referente ao resultado obtido
     */
    listPedidosFeitos() {
        return function (req, res) {
            const id_pedido = req.query.id_pedido;
            const id_user = req.query.id_user;

            pedidos.listPedidosFeitos(id_pedido, id_user).then((response) => {
                if (response.error) res.status(response.code).send({ error: response.error });
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

/**
 * Lista a quantidade de todos os produtos em um pedido
 * @returns mensagem referente ao resultado obtido
 */
// setupSocketListeners(socket) {
//     socket.on('listQtdeTotalProdutos', (data) => {
//         const { id_user } = data;

//         if (!id_user) {
//             socket.emit('listQtdeTotalProdutosResponse', { error: "ID do usuário é obrigatório." });
//             return;
//         }

//         pedidos.listQtdeTotalProdutos(id_user, (error, response) => {
//             if (error) {
//                 socket.emit('listQtdeTotalProdutosResponse', { error: error });
//             } else {
//                 socket.emit('listQtdeTotalProdutosResponse', response);
//             }
//         });
//     });
// }
listQtdeTotalProdutos(id_user, callback) {
    if (!db) {
        return callback({ error: "Banco de dados não configurado." });
    }

    db.query(
        `SELECT 
        SUM(pedidos.quantidade) AS quantidade_produtos,
        SUM(CASE WHEN produtos.categoria = 1 THEN pedidos.quantidade ELSE 0 END) AS quantidade_produtos_suino,
        SUM(CASE WHEN produtos.categoria = 2 THEN pedidos.quantidade ELSE 0 END) AS quantidade_produtos_bovino
        FROM pedidos
        JOIN produtos ON pedidos.id_produto = produtos.id
        WHERE 
        pedidos.id_user = ? AND 
        pedidos.concluido = false`,
        [id_user],
        (error, result) => {
            if (error) {
                console.log(error);
                return callback({ error: "Erro ao consultar a quantidade total de produtos." });
            }

            if (result.length === 0 || (result[0].quantidade_produtos_2 === null && result[0].quantidade_produtos_1 === null && result[0].quantidade_total)) {
                return callback(null, { quantidade_produtos_suino: 0, quantidade_produtos_bovino: 0, quantidade_total: 0 });
            }

            return callback(null, result[0]);
        }
    );
}

// Função para configurar os listeners de socket
setupSocketListeners(socket) {
    socket.on('listQtdeTotalProdutos', (data) => {
        const { id_user } = data;

        if (!id_user) {
            socket.emit('listQtdeTotalProdutosResponse', { error: "ID do usuário é obrigatório." });
            return;
        }

        this.listQtdeTotalProdutos(id_user, (error, response) => {
            if (error) {
                socket.emit('listQtdeTotalProdutosResponse', { error: error });
            } else {
                socket.emit('listQtdeTotalProdutosResponse', response);
            }
        });
    });
}

    /**
     * Delete um usuário
     * @returns mensagem referente ao resultado obtido
     */
    delete() {
        return function (req, res) {
            const { id } = req.params;
            
            pedidos.delete(id).then((response) => {
                if (response.error) res.status(response.code).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

    
    /**
     * Delete um usuário
     * @returns mensagem referente ao resultado obtido
     */
    deletePedidoByUserId() {
        return function (req, res) {
            const { id } = req.params;
            
            pedidos.deletePedidoByUserId(id).then((response) => {
                if (response.error) res.status(response.code).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

        /**
     * Delete um usuário
     * @returns mensagem referente ao resultado obtido
     */
        deleteProdutoDoPedido() {
            return function (req, res) {
                const id_produto  = req.query.id_produto;
                const id_user  = req.query.id_user;

                
                pedidos.deleteProdutoDoPedido(id_produto, id_user).then((response) => {
                    if (response.error) res.status(response.code).send(response);
                    else return res.status(200).send(response);
                }).catch((error) => {
                    throw new Error(error);
                });
            }
        }

    /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
    update() {
        return function (req, res) {
            const { data } = req.body;
            
            pedidos.update(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }

        /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
        concluirPedido() {
            return function (req, res) {
                const data = req.body;
                
                pedidos.concluirPedido(data).then((response) => {
                    if (response.error) res.status(400).send(response);
                    else return res.status(200).send(response);
                }).catch((error) => {
                    throw new Error(error);
                });
            }
        }

    /**
     * Atualiza os dados de um usuário
     * @returns mensagem referente ao resultado obtido
     */
    updateQuantidade() {
        return function (req, res) {
            const  data  = req.body;
            
            pedidos.updateQuantidade(data).then((response) => {
                if (response.error) res.status(400).send(response);
                else return res.status(200).send(response);
            }).catch((error) => {
                throw new Error(error);
            });
        }
    }
  
}

module.exports = pedidosController;